/*-
 * Copyright (C)2010..2024 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 * Used tcpmssd (c) 2000 by Ruslan Ermilov as reference.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2010..2024 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: cfg.c,v 1.3 2024/03/16 15:32:14 babolo Exp $"

#define BLIN_COMPAT  4
#define Bpars_COMPAT 4
#define MIFE_COMPAT  5

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sysexits.h>
#include <strings.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include <mife.h>
#include "mssseparated.h"

void
mssseparated_printcfg(mssseparated_icfg const *icfg) {
    FILE *ou;

    if  (!icfg) {
        ifBLIN_QX0("Config absent");
        errno = EDOOFUS;
        goto out;
    }
#   define blin_internal_flags (icfg->flags)
    if  (!icfg->logfi) {
        ou = stderr;
    } else if (!(ou = fopen(icfg->logfi, "a"))) {
        ifBLIN_QW0("fopen=%s~", icfg->logfi);
        goto out;
    }
    if  (0 > fflush(NULL)) {
        ifBLIN_QW0("fflush NULL");
        goto out;
    }
    fprintf(ou, "# top->%u\n", icfg->top);
    for (u_int32_t i = 0; i < icfg->top; i++) {
        if  (icfg->entry[i].min == MSSSEPARATE_MIIN) {
            if  (icfg->entry[i].max == 0xFFFF) {
            ;   if  (icfg->entry[i].rule < 0) {
            ;       fprintf(ou, "                  # %2u\n", i);
            ;   } else {
            ;       fprintf(ou, "            %5d # %2u\n", icfg->entry[i].rule, i);
            ;   }
            } else if (icfg->entry[i].min == icfg->entry[i].max) {
            ;   if  (icfg->entry[i].rule < 0) {
            ;       fprintf(ou, "%d                # %2u\n", icfg->entry[i].min, i);
            ;   } else {
            ;       fprintf(ou, "%d          %5d # %2u\n", icfg->entry[i].min, icfg->entry[i].rule, i);
            ;   }
            } else {
            ;   if  (icfg->entry[i].rule < 0) {
            ;       fprintf( ou, "%d-%d   %s       # %2u\n"
                           , icfg->entry[i].min, icfg->entry[i].max
                           , ( (icfg->entry[i].max < 100)
                             ? ( (icfg->entry[i].max < 10)
                               ? ((icfg->entry[i].max < 0) ? "   " : "    ")
                               : "   "
                               )
                             : ( (icfg->entry[i].max < 10000)
                               ? ((icfg->entry[i].max < 1000) ? "  " : " ")
                               : ""
                             ) )
                           , i
                           );
            ;   } else {
            ;       fprintf( ou, "%d-%d   %s %5d # %2u\n"
                           , icfg->entry[i].min, icfg->entry[i].max
                           , ( (icfg->entry[i].max < 100)
                             ? ( (icfg->entry[i].max < 10)
                               ? ((icfg->entry[i].max < 0) ? "   " : "    ")
                               : "   "
                               )
                             : ( (icfg->entry[i].max < 10000)
                               ? ((icfg->entry[i].max < 1000) ? "  " : " ")
                               : ""
                             ) )
                           , icfg->entry[i].rule
                           , i
                           );
            }   }
        } else {
            if  (icfg->entry[i].min == icfg->entry[i].max) {
            ;   if  (icfg->entry[i].rule < 0) {
            ;       fprintf( ou, "%d      %s       # %2u\n"
                           , icfg->entry[i].min
                           , ( (icfg->entry[i].min < 100)
                             ? ( (icfg->entry[i].min < 10)
                               ? ((icfg->entry[i].min < 0) ? "   " : "    ")
                               : "   "
                               )
                             : ( (icfg->entry[i].min < 10000)
                               ? ((icfg->entry[i].min < 1000) ? "  " : " ")
                               : ""
                             ) )
                           , i
                           );
            ;   } else {
            ;       fprintf( ou, "%d      %s %5d # %2u\n"
                           , icfg->entry[i].min
                           , ( (icfg->entry[i].min < 100)
                             ? ( (icfg->entry[i].min < 10)
                               ? ((icfg->entry[i].min < 0) ? "   " : "    ")
                               : "   "
                               )
                             : ( (icfg->entry[i].min < 10000)
                               ? ((icfg->entry[i].min < 1000) ? "  " : " ")
                               : ""
                             ) )
                           , icfg->entry[i].rule
                           , i
                           );
            ;   }
            } else {
            ;   if  (icfg->entry[i].rule < 0) {
            ;       fprintf( ou, "%d-%d%s%s       # %2u\n"
                           , icfg->entry[i].min, icfg->entry[i].max
                           , ( (icfg->entry[i].min < 100)
                             ? ( (icfg->entry[i].min < 10)
                               ? ((icfg->entry[i].min < 0) ? "   " : "    ")
                               : "   "
                               )
                             : ( (icfg->entry[i].min < 10000)
                               ? ((icfg->entry[i].min < 1000) ? "  " : " ")
                               : ""
                             ) )
                           , ( (icfg->entry[i].max < 100)
                             ? ( (icfg->entry[i].max < 10)
                               ? ((icfg->entry[i].max < 0) ? "   " : "    ")
                               : "   "
                               )
                             : ( (icfg->entry[i].max < 10000)
                               ? ((icfg->entry[i].max < 1000) ? "  " : " ")
                               : ""
                             ) )
                           , i
                           );
            ;   } else {
            ;       fprintf( ou, "%d-%d%s%s %5d # %2u\n"
                           , icfg->entry[i].min, icfg->entry[i].max
                           , ( (icfg->entry[i].min < 100)
                             ? ( (icfg->entry[i].min < 10)
                               ? ((icfg->entry[i].min < 0) ? "   " : "    ")
                               : "   "
                               )
                             : ( (icfg->entry[i].min < 10000)
                               ? ((icfg->entry[i].min < 1000) ? "  " : " ")
                               : ""
                             ) )
                           , ( (icfg->entry[i].max < 100)
                             ? ( (icfg->entry[i].max < 10)
                               ? ((icfg->entry[i].max < 0) ? "   " : "    ")
                               : "   "
                               )
                             : ( (icfg->entry[i].max < 10000)
                               ? ((icfg->entry[i].max < 1000) ? "  " : " ")
                               : ""
                             ) )
                           , icfg->entry[i].rule
                           , i
                           );
    }   }   }   }
    if  (0 > fflush(ou)) ifBLIN_QW0("fflush");
out:
    return;
#   undef blin_internal_flags
}

mssseparated_icfg *
mssseparated_readcfg(mssseparated_icfg *icfg) {
#   define blin_internal_flags (icfg->flags)
    u_int32_t          bpfl;
    ssize_t            clen;
    char              *cfg ;
    babolo_parm       *bp  = NULL;
    char              *c   ;

    if  (!icfg->confi || !*icfg->confi) {
        ifBLIN_QX0("Config file name absent");
        errno = EDOOFUS;
        goto out;
    }
    if  (!(icfg->md = mife_init(MIFE_FULL))) {
        ifBLIN_QW0("mife_init");
        goto out;
    }
    if  (0 > mife_ctlfile(icfg->md, icfg->confi)) {
        ifBLIN_QW0("mife_ctlfile");
        goto out;
    }
    if  (0 > (clen = mife_read(icfg->md, (ssize_t)0, (off_t)0))) {
        ifBLIN_QW0("mife_read");
        goto out;
    }
    if  (!(cfg = mife_get(icfg->md, (off_t)0))) {
        ifBLIN_QW0("mife_get");
        goto out;
    }
    for (icfg->top = 0; !!cfg && !!*cfg; ) {
        if  (*cfg == '#') {
            cfg = index(cfg, '\n');
            continue;
        }
        if  (icfg->top >= icfg->ilen) {
            icfg->ilen *= 2;
            icfg = reallocf( icfg
                           , icfg->ilen * sizeof(mssseparated_icfgentry) + sizeof(mssseparated_icfg)
                           );
            if  (!icfg) {
                ifBLIN_QW0("reallocf");
                goto out;
        }   }
        bpfl = Bpars_CRLN | Bpars_LFLN | Bpars_FAST | Bpars_FIXB | Bpars_NOAL | Bpars_FIRM;
        if  (!(icfg->flags & MSSSEPARATE_CFGENV)) bpfl |= Bpars_NOEN;
        bp = babolo_getparm(bpfl | 3, &cfg, "#", 0);
        if  (bp->argc <= 1 && !bp->argv[0]) continue;
        icfg->entry[icfg->top].min = MSSSEPARATE_MIIN;
        icfg->entry[icfg->top].max = 0xFFFF;
        icfg->entry[icfg->top].rule = -1;
        if  (/* bp->argc && (already) */ !!(c = bp->argv[0]) && !!*c) {
            icfg->entry[icfg->top].min = (int32_t)strtol(c, &c, 0);
            if  (*c == '-') {
                icfg->entry[icfg->top].max = (int32_t)strtol(++c, &c, 0);
            } else {
                icfg->entry[icfg->top].max = icfg->entry[icfg->top].min;
        }   }
        if  (icfg->entry[icfg->top].min < MSSSEPARATE_MIIN)
            icfg->entry[icfg->top].min = MSSSEPARATE_MIIN;
        if  (icfg->entry[icfg->top].min > 0xFFFF)
            icfg->entry[icfg->top].min = 0xFFFF;
        if  (icfg->entry[icfg->top].max < MSSSEPARATE_MIIN)
            icfg->entry[icfg->top].max = MSSSEPARATE_MIIN;
        if  (icfg->entry[icfg->top].max > 0xFFFF)
            icfg->entry[icfg->top].max = 0xFFFF;
        if  ((bp->argc > 1) && !!(c = bp->argv[1]) && !!*c) {
            icfg->entry[icfg->top].rule = (int32_t)strtol(c, &c, 0);
        }
        if  (icfg->entry[icfg->top].rule < -1) icfg->entry[icfg->top].rule = -1;
        if  (icfg->entry[icfg->top].rule > 0x0FFFF) icfg->entry[icfg->top].rule = 0x0FFFF;
        icfg->top++;
    }
out:
    if  (!!bp) babolo_freeparm(bp);
    return(icfg);
#   undef blin_internal_flags
}
