/*-
 * Copyright (C)2010..2024 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 * Used tcpmssd (c) 2000 by Ruslan Ermilov as reference.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: mssseparated.h,v 1.3 2024/03/16 15:32:14 babolo Exp $
 */

#define MSSSEPARATE_FLOG  "/var/log/mssseparated"
#define MSSSEPARATE_FPID  "/var/run/mssseparated.pid"
#define MSSSEPARATE_FCONF "/usr/local/etc/mssseparated.conf"
#define MSSSEPARATE_MIIN  (-8)
#define MSSSEPARATE_ILEN  340

typedef struct {
    int32_t min;
    int32_t max;
    int32_t rule;
} mssseparated_icfgentry;

typedef struct {
    BLIN_flag              flags   ;
#       define MSSSEPARATE_SLEEP  0x0100 /*                 */
#       define MSSSEPARATE_CFGENV 0x0200 /*         */
#       define MSSSEPARATE_DAEMON 0x0400 /*                               */
#       define MSSSEPARATE_OUCONF 0x0800 /*                        */
#       define MSSSEPARATE_CTLDMP 0x1000 /*  blin_ctl(BLIN_CTL_DUMP) */
    u_int32_t              ilen    ;
    u_int32_t              top     ;
    unsigned int           seconds ;
    u_int16_t              port    ;
    int16_t                dummy   ;
    int32_t                dummy0  ;
    FILE                  *logg    ;
    char const            *confi   ;
    char const            *pidfi   ;
    char const            *logfi   ;
    mife_descriptor       *md      ;
    mssseparated_icfgentry entry   [0];
} mssseparated_icfg;

typedef struct {
        u_short th_sport;               /* source port */
        u_short th_dport;               /* destination port */
        u_short th_seq[2];              /* sequence number */
        u_short th_ack[2];              /* acknowledgement number */
#if BYTE_ORDER == LITTLE_ENDIAN
        u_char  th_x2:4,                /* (unused) */
                th_off:4;               /* data offset */
#endif
#if BYTE_ORDER == BIG_ENDIAN
        u_char  th_off:4,               /* data offset */
                th_x2:4;                /* (unused) */
#endif
        u_char  th_flags;
#define TH_FIN  0x01
#define TH_SYN  0x02
#define TH_RST  0x04
#define TH_PUSH 0x08
#define TH_ACK  0x10
#define TH_URG  0x20
#define TH_ECE  0x40
#define TH_CWR  0x80
#define TH_FLAGS        (TH_FIN|TH_SYN|TH_RST|TH_PUSH|TH_ACK|TH_URG|TH_ECE|TH_CWR)
#define PRINT_TH_FLAGS  "\20\1FIN\2SYN\3RST\4PUSH\5ACK\6URG\7ECE\10CWR"

        u_short th_win;                 /* window */
        u_short th_sum;                 /* checksum */
        u_short th_urp;                 /* urgent pointer */
} tcpshdr;

void               mssseparated_printcfg __P((mssseparated_icfg const *icfg));
mssseparated_icfg *mssseparated_readcfg  __P((mssseparated_icfg *icfg));
